#import "FireMethodFromLua.h"
#import <UIKit/UIKit.h>
#include <CoronaRuntime.h>
#import <Skillz/Skillz.h>

@interface SzDelegate: NSObject <SkillzDelegate>
@property (nonatomic) Corona::Lua::Ref lstRef;
@property (nonatomic) lua_State *L;
@end

@implementation SzDelegate

- (void)sendEventWithPhase:(const char *)phase
               withIsError:(bool)isError
                  withGameParameters:(const char*)gameParameters
                  withMatchInfo:(const char*)matchInfo
{
    
    CoronaLuaNewEvent( self.L, "skillz");
    
    lua_pushstring( self.L, phase);
    lua_setfield( self.L, -2, "phase" );
    
    lua_pushboolean(self.L, isError);
    lua_setfield( self.L, -2, "isError" );
    
    lua_pushstring(self.L, gameParameters);
    lua_setfield( self.L, -2, "gameParameters" );
    
    lua_pushstring(self.L, matchInfo);
    lua_setfield( self.L, -2, "matchInfo" );

    CoronaLuaDispatchEvent( self.L, self.lstRef, 0 );
    
}

- (void)tournamentWillBegin:(NSDictionary *)gameParameters withMatchInfo:(SKZMatchInfo *)matchInfo {
    
    NSString* name =  matchInfo.name;
    NSString* matchDescription =  matchInfo.matchDescription;
    
    lua_newtable(self.L);
    lua_pushstring(self.L, [name cStringUsingEncoding:NSASCIIStringEncoding]);
    lua_setfield( self.L, -2, "matchId" );
    
    lua_pushstring(self.L, [matchDescription cStringUsingEncoding:NSASCIIStringEncoding]);
    lua_setfield( self.L, -2, "matchToken" );
    
    
    [self sendEventWithPhase:"willBegin" withIsError:NO withGameParameters:lua_tostring(self.L, -1) withMatchInfo:"" ];
    
    
}

- (void)skillzHasFinishedLaunching {
    [self sendEventWithPhase:"finishedLaunching" withIsError:NO withGameParameters:"" withMatchInfo:""];
}

- (void)skillzWillExit {
    [self sendEventWithPhase:"willExit" withIsError:NO withGameParameters:"" withMatchInfo:""];
    
}

- (void)userDidDeposit:(NSString *)userId cashAmount:(NSNumber *)cashAmount bonusCashAmount:(NSNumber *)bonusCashAmount promoAmount:(NSNumber *)promoAmount currencyCode:(NSString *)currencyCode {
    
    [self sendEventWithPhase:"didDeposit" withIsError:NO withGameParameters:"" withMatchInfo:""];
          
    
    try {
        
        //[Tapjoy setUserCohortVariable:1 value:@"Paying"];
        
    } catch (NSException* ex) {
        NSLog(@"**** Did deposit error: %@", ex.debugDescription);
        
    }
    
    
}

@end

static SzDelegate* szDelegate = [[SzDelegate alloc] init];
//static InterDelegate* interDelegate = [[InterDelegate alloc] init];

class FireMethodFromLua
{
    public:
        typedef FireMethodFromLua Self;
        
    public:
        static const char kName[];
        static const char kEvent[];
        
    protected:
        FireMethodFromLua();
        
    public:
        bool Initialize( CoronaLuaRef listener );
        
    public:
        CoronaLuaRef GetListener() const { return fListener; }
        
    public:
        static int Open( lua_State *L );
        
    protected:
        static int Finalizer( lua_State *L );
        
    public:
        static Self *ToLibrary( lua_State *L );
        
    public:
        static int init( lua_State *L );
        static int doSomeWork( lua_State *L );
        
    private:
        CoronaLuaRef fListener;
    
};


const char FireMethodFromLua::kName[] = "FireMethodFromLua";
const char FireMethodFromLua::kEvent[] = "FireMethodFromLuaEvent";

FireMethodFromLua::FireMethodFromLua()
:    fListener( NULL )
{
}


bool
FireMethodFromLua::Initialize( CoronaLuaRef listener )
{
    // Can only initialize listener once
    bool result = ( NULL == fListener );
    
    if ( result )
    {
        fListener = listener;
    }
    
    return result;
}

int
FireMethodFromLua::Open( lua_State *L )
{
    // Register __gc callback
    const char kMetatableName[] = __FILE__; // Globally unique string to prevent collision
    CoronaLuaInitializeGCMetatable( L, kMetatableName, Finalizer );
    
    // Functions in library
    const luaL_Reg kVTable[] =
    {
        { "init", init },
        { "doSomeWork", doSomeWork },
        { NULL, NULL }
    };
    
    // Set library as upvalue for each library function
    Self *library = new Self;
    CoronaLuaPushUserdata( L, library, kMetatableName );
    
    luaL_openlib( L, kName, kVTable, 1 ); // leave "library" on top of stack
    
    return 1;
}

int
FireMethodFromLua::Finalizer( lua_State *L )
{
    Self *library = (Self *)CoronaLuaToUserdata( L, 1 );
    
    CoronaLuaDeleteRef( L, library->GetListener() );
    
    delete library;
    
    return 0;
}

FireMethodFromLua *
FireMethodFromLua::ToLibrary( lua_State *L )
{
    // library is pushed as part of the closure
    Self *library = (Self *)CoronaLuaToUserdata( L, lua_upvalueindex( 1 ) );
    return library;
}

int
FireMethodFromLua::init( lua_State *L )
{
    int listenerIndex = 1;
    
    if ( CoronaLuaIsListener( L, listenerIndex, kEvent ) )
    {
        Self *library = ToLibrary( L );
        
        CoronaLuaRef listener = CoronaLuaNewRef( L, listenerIndex );
        library->Initialize( listener );
        
        szDelegate.L = L;
        szDelegate.lstRef = listener;
    }
    
    return 0;
}


// [Lua] FireMethodFromLua.doSomeWork()
int
FireMethodFromLua::doSomeWork( lua_State *L )
{
    
    const char *workType = lua_tostring( L, 1 );
    const char *action = "";
    const char *stringParameter = "";
    NSNumber *numberParameter = 0;
    NSNumber *numberParameter2 = 0;
    bool booleanParameter = NO;
    
    
    if ( lua_type( L, -1 ) == LUA_TTABLE )
    {
        
        //String parameters
        lua_getfield( L, -1, "action" );
        if ( lua_type( L, -1 ) == LUA_TSTRING )
        {
            action = lua_tostring( L, -1 );
        }
        lua_pop( L, 1 );
        
        lua_getfield( L, -1, "stringParameter" );
        if ( lua_type( L, -1 ) == LUA_TSTRING )
        {
           stringParameter = lua_tostring( L, -1 );
        }
       
        
        //Number params
        lua_pop( L, 1 );
        
        lua_getfield( L, -1, "numberParameter" );
        if ( lua_type( L, -1 ) == LUA_TNUMBER )
        {
            numberParameter = [NSNumber numberWithLong:lua_tointeger( L, -1 )];
        }
        
        lua_pop( L, 1 );
        
        lua_getfield( L, -1, "numberParameter2" );
        if ( lua_type( L, -1 ) == LUA_TNUMBER )
        {
            numberParameter2 = [NSNumber numberWithLong:lua_tointeger( L, -1 )];
        }
        
        lua_pop( L, 1 );
        
        //Bool params
        lua_getfield( L, -1, "booleanParameter" );
        if ( lua_type( L, -1 ) == LUA_TBOOLEAN )
        {
            booleanParameter = lua_toboolean( L, -1 );
        }
        
        lua_pop( L, 1 );
        
       
    }
    
    if ( strcmp(workType, "skillz") == 0){
        
        lua_pushnil(L);
        
        if (strcmp(action, "launchSkillzUI") == 0){
             [[Skillz skillzInstance] launchSkillz];
            
        } else if (strcmp(action, "abort") == 0) {
            [[Skillz skillzInstance] notifyPlayerAbortWithCompletion:^(void)
            {
                
            }];
        
        } else if (strcmp(action, "tournamentResult") == 0) {
            [[Skillz skillzInstance] displayTournamentResultsWithScore: numberParameter withCompletion:^(void)
            {
                
            }];
            
        } else if (strcmp(action, "getRandomNumber") == 0
                   && numberParameter != 0) {
            
            lua_pop(L, 1);
            lua_pushnumber( L, [Skillz getRandomNumberWithMin:1 andMax:numberParameter.integerValue]);
            
        } else if (strcmp(action, "init") == 0
                   && strcmp(stringParameter, "") != 0) {
            
            SkillzEnvironment se = SkillzSandbox;
            if (booleanParameter == YES) {
                se = SkillzProduction;
                
            }
            
            [[Skillz skillzInstance] initWithGameId:[NSString stringWithUTF8String: stringParameter]
                forDelegate:szDelegate
                withEnvironment:se
                allowExit:true
             
             ];
        
        } else if (strcmp(action, "updateScore") == 0
            && numberParameter != 0){
            [[Skillz skillzInstance] updatePlayersCurrentScore:numberParameter];
            
            
        }
        
    } else if (strcmp(workType, "ads") == 0 ) {
 
        
    }
    
    return 1;
}

// ----------------------------------------------------------------------------

CORONA_EXPORT int luaopen_FireMethodFromLua( lua_State *L )
{
    return FireMethodFromLua::Open( L );
}
